unit ConfFile;

interface

uses
  Windows;

type
  PPCharArray = ^TPCharArray;
  TPCharArray = array[0..MaxInt div SizeOf(Pointer)-1] of PChar;

  TConfLine = record
    Name: PChar;
    Args: array[0..63] of PChar;
    ArgCount: Integer;
  end;

  TConfFile = class(TObject)
  private
    FNameBuf: array[0..1023] of Char;
    FArgBuf: array[0..1023] of Char;
    FBuffer: string;
    FBufferPos: Integer;
    ResultCode: Integer;
  public
    constructor Create(const FileName: string);
    function GetLine(var ConfLine: TConfLine): Boolean;
  end;

function GetProfDir(const Name: string): string;
function GetArgs(InBuf: PChar; NameBuf: PChar; NameBufSize: Integer;
  ArgBuf: PChar; ArgBufSize: Integer; Args: PPCharArray): Integer;
function StrToRGB(p: PChar): COLORREF;
procedure LoadFont(const ConfLine: TConfLine; var LogFont: TLogFont);

implementation

function GetProfDir(const Name: string): string;
var
  s: string;
  buf: array[0..MAX_PATH] of Char;
  p: PChar;
begin
  Result := '';
  // R}hCŎw肳ꂽꍇ
  s := ParamStr(1);
  if Copy(s, 1, 6) = '/prof=' then
  begin
    Result := Copy(s, 7, Length(s)-6);
    if Result[Length(Result)] <> '\' then
      Result := Result + '\';
  end;
  // ϐ
  if Result = '' then
  begin
    FillChar(buf, SizeOf(buf), 0);
    GetEnvironmentVariable('NKDTPROF', buf, SizeOf(buf));
    if buf <> '' then
    begin
      Result := buf;
      if Result[Length(Result)] <> '\' then
        Result := Result + '\';
      Result := Result + Name + '\';
    end;
  end;
  // ftHgexeƓꏊ
  if Result = '' then
  begin
    s := ParamStr(0);
    GetFullPathName(PChar(s), SizeOf(buf), buf, p);
    p^ := #0;
    Result := buf;
  end;
end;

function LoadFile(const FileName: string; var Buffer: string): Integer;
var
  F: Text;
  Buf: array[0..1023] of Char;
  NameBuf: array[0..1023] of Char;
  ArgBuf: array[0..1023] of Char;
  Args: array[0..63] of PChar;
  TextBuf: array[0..4095] of Char;
  s: string;
begin
  AssignFile(F, FileName);
  {$I-}
  Reset(F);
  {$I+}
  Result := IOResult;
  if Result <> 0 then Exit;
  SetTextBuf(F, TextBuf);
  while not Eof(F) do
  begin
    Readln(F, s);
    if s = '' then Continue;
    if s[1] = '#' then Continue;
    lstrcpyn(Buf, PChar(s), SizeOf(Buf));
    GetArgs(Buf, NameBuf, SizeOf(NameBuf), ArgBuf, SizeOf(ArgBuf), @Args);
    if NameBuf = '@Include' then
    begin
      if Args[0] <> '' then LoadFile(Args[0], Buffer);
    end else
      Buffer := Buffer + s + #10;
  end;
  CloseFile(F);
end;

constructor TConfFile.Create(const FileName: string);
begin
  ResultCode := LoadFile(FileName, FBuffer);
end;

function TConfFile.GetLine(var ConfLine: TConfLine): Boolean;
var
  Buf: array[0..1023] of Char;
  i: Integer;
begin
  if ResultCode <> 0 then
  begin
    Result := False;
    Exit;
  end;
  if FBufferPos = Length(FBuffer) then
  begin
    Result := False;
    Exit;
  end;
  i := 0;
  Inc(FBufferPos);
  while (FBuffer[FBufferPos] <> #10) and (i < (SizeOf(Buf)-1)) do
  begin
    Buf[i] := FBuffer[FBufferPos];
    Inc(FBufferPos);
    Inc(i);
  end;
  Buf[i] := #0;
  ConfLine.ArgCount := GetArgs(Buf, FNameBuf, SizeOf(FNameBuf),
    FArgBuf, SizeOf(FArgBuf), @ConfLine.Args);
  ConfLine.Name := FNameBuf;
  Result := True;
end;

// 擪̔Cӌspace폜Ď̔Cӌspace̒̕Ԃ
function SplitSpace(var p: PChar): PChar;
var
  q: PChar;
begin
  while (p^ = ' ') or (p^ = #9) do
    Inc(p);
  q := p;
  while (q^ <> ' ') and (q^ <> #9) and (q^ <> #0) do
    Inc(q);
  if q^ = #0 then
  begin
    Result := q;
    Exit;
  end;
  q^ := #0;
  Inc(q);
  Result := q;
end;

// 'Name Param, Param, "Param", Param, ...'
// ȕ𕪊ĈPCharȃ|C^̃XgԂ
//  ""ɊĂȂspaceƃ^uׂ͂Ď菜
// ret: ̐
function GetArgs(InBuf: PChar; NameBuf: PChar; NameBufSize: Integer;
   ArgBuf: PChar; ArgBufSize: Integer; Args: PPCharArray): Integer;
var
  p, q: PChar;
  sp: Char;
  n: Integer;
  qflg: Boolean;
begin
  FillChar(ArgBuf^, ArgBufSize, 0);

  p := SplitSpace(InBuf); // p: start of arguments
  lstrcpyn(NameBuf, InBuf, NameBufSize);
  if p^ = #0 then
  begin
    Result := 0;
    Exit;
  end;

  // ""ɊĂȂspaceƃ^uׂ͂Ď菜
  n := 0;
  q := ArgBuf;
  qflg := False;
  while (p^ <> #0) and (n < ArgBufSize) do
  begin
    if p^ = '"' then
      qflg := not qflg;
    if qflg or ((p^ <> ' ') and (p^ <> #9)) then
    begin
      q^ := p^;
      Inc(q);
      Inc(n);
    end;
    Inc(p);
  end;

  p := ArgBuf;
  n := 0;
  sp := #0;
  repeat
    q := p;
    if p^ = '"' then
    begin
      Inc(p);
      Inc(q);
      while (p^ <> '"') and (p^ <> #0) do
        Inc(p);
      if p^ = #0 then
        Break;
      p^ := #0;
      Inc(p);
    end;
    while (p^ <> ',') and (p^ <> #0) do
      Inc(p);
    sp := p^;
    p^ := #0;
    Args[n] := q;
    Inc(n);
    Inc(p);
  until sp <> ',';
  Result := n;
end;

// '#FFFFFF' `̕񂩂 COLORREF l
function StrToRGB(p: PChar): COLORREF;
var
  col, code: Integer;
begin
  if p = nil then
    Result := -1
  else if p^ = '#' then
  begin
    p^ := '$';
    Val(p, col, code);
    Result := ((col shl 16) and $FF0000) or (col and $00FF00) or (col shr 16);
  end else
    Result := -1;
end;

function StrToInt(const s: string): Integer;
var
  code: Integer;
begin
  Val(s, Result, code);
end;

// Height, FontName, BoldFlg, ItalicFlg, CharSet
// ArgsLOGFONTԂ
procedure LoadFont(const ConfLine: TConfLine; var LogFont: TLogFont);
var
  i: Integer;
begin
  FillChar(LogFont, SizeOf(LogFont), 0);
  with LogFont do begin
    lfOutPrecision   := OUT_DEFAULT_PRECIS;
    lfClipPrecision  := CLIP_DEFAULT_PRECIS;
    lfQuality        := DRAFT_QUALITY;
    lfPitchAndFamily := DEFAULT_PITCH;
    lfWeight := FW_NORMAL;
    lfCharSet := DEFAULT_CHARSET;
    for i := 0 to ConfLine.ArgCount - 1 do
    begin
      case i of
        0: lfHeight := StrToInt(ConfLine.Args[i]);
        1: lstrcpyn(lfFaceName, ConfLine.Args[i], SizeOf(lfFaceName));
        2: if StrToInt(ConfLine.Args[i]) <> 0 then lfWeight := FW_BOLD;
        3: lfItalic := StrToInt(ConfLine.Args[i]);
        4: lfCharSet := StrToInt(ConfLine.Args[i]);
      end;
    end;
  end;
end;


end.
