unit keytable;

interface

uses Windows, ConfFile;

type

TKey = record
  Alt: Boolean;
  Shift: Boolean;
  Ctrl: Boolean;
  VKey: Integer;
  CmdId: Integer;
end;

TCmd = record
  Name: string;
  Caption: string;
  Id: Integer;
end;

PCmdTable = ^TCmdTable;
TCmdTable = array[0..0] of TCmd;

TKeyTable = class(TObject)
protected
  FKeyList: array[0..255] of TKey;
  FKeyListCount: Integer;
public
  CmdTable: PCmdTable;
  MenuHandle: HMENU;
  function StrToKey(const KeyName: string; const ActionName: string): TKey;
  function KeyToStr(const Key: TKey): string;
  procedure LoadFromFile(const FileName: string);
  function GetCmdId(Alt, Shift, Ctrl: Boolean; VKey: Integer): Integer;
  function IdToKeyNames(Id: Integer): string;
  procedure AddMenuItem(Id: Integer);
  procedure SetDefaultKeys(const DefaultKeys: array of string);
end;

implementation

const
  SpKeys: array[0..41] of TCmd = (
    (Name: 'BS'; Id: VK_BACK),
    (Name: 'Tab'; Id: VK_TAB),
    (Name: 'Enter'; Id: VK_RETURN),
    (Name: 'Esc'; Id: VK_ESCAPE),
    (Name: 'Space'; Id: VK_SPACE),
    (Name: 'PgUp'; Id: VK_PRIOR),
    (Name: 'PgDn'; Id: VK_NEXT),
    (Name: 'End'; Id: VK_END),
    (Name: 'Home'; Id: VK_HOME),
    (Name: 'Left'; Id: VK_LEFT),
    (Name: 'Up'; Id: VK_UP),
    (Name: 'Right'; Id: VK_RIGHT),
    (Name: 'Down'; Id: VK_DOWN),
    (Name: 'Ins'; Id: VK_INSERT),
    (Name: 'Del'; Id: VK_DELETE),
    (Name: 'Num0'; Id: VK_NUMPAD0),
    (Name: 'Num1'; Id: VK_NUMPAD1),
    (Name: 'Num2'; Id: VK_NUMPAD2),
    (Name: 'Num3'; Id: VK_NUMPAD3),
    (Name: 'Num4'; Id: VK_NUMPAD4),
    (Name: 'Num5'; Id: VK_NUMPAD5),
    (Name: 'Num6'; Id: VK_NUMPAD6),
    (Name: 'Num7'; Id: VK_NUMPAD7),
    (Name: 'Num8'; Id: VK_NUMPAD8),
    (Name: 'Num9'; Id: VK_NUMPAD9),
    (Name: 'Num*'; Id: VK_MULTIPLY),
    (Name: 'Num+'; Id: VK_ADD),
    (Name: 'Num-'; Id: VK_SUBTRACT),
    (Name: 'Num.'; Id: VK_DECIMAL),
    (Name: 'Num/'; Id: VK_DIVIDE),
    (Name: 'F1'; Id: VK_F1),
    (Name: 'F2'; Id: VK_F2),
    (Name: 'F3'; Id: VK_F3),
    (Name: 'F4'; Id: VK_F4),
    (Name: 'F5'; Id: VK_F5),
    (Name: 'F6'; Id: VK_F6),
    (Name: 'F7'; Id: VK_F7),
    (Name: 'F8'; Id: VK_F8),
    (Name: 'F9'; Id: VK_F9),
    (Name: 'F10'; Id: VK_F10),
    (Name: 'F11'; Id: VK_F11),
    (Name: 'F12'; Id: VK_F12)
  );

procedure TKeyTable.AddMenuItem(Id: Integer);
begin
  if IsMenu(MenuHandle) then
    AppendMenu(MenuHandle, MF_STRING, Id, PChar(IdToKeyNames(Id)));
end;

function TKeyTable.IdToKeyNames(Id: Integer): string;
var
  i, j: Integer;
  s: string;
begin
  i := 0;
  Result := '';
  while CmdTable[i].Name <> '' do
  begin
    if CmdTable[i].Id = Id then
    begin
      for j := 0 to FKeyListCount - 1 do
      begin
        if CmdTable[i].Id = FKeyList[j].CmdId then
        begin
          if s = '' then
            s := KeyToStr(FKeyList[j])
          else
            s := s + ', ' + KeyToStr(FKeyList[j]);
        end;
      end;
      Result := CmdTable[i].Caption + #9 + s;
      Exit;
    end;
    Inc(i);
  end;
end;

procedure TKeyTable.LoadFromFile(const FileName: string);
var
  ConfFile: TConfFile;
  ConfLine: TConfLine;
  i: Integer;
begin
  ConfFile := TConfFile.Create(FileName);
  i := 0;
  while ConfFile.GetLine(ConfLine) do
  begin
    Inc(i);
    FKeyList[i-1] := StrToKey(ConfLine.Name, ConfLine.Args[0]);
    if (i-1) = High(FKeyList) then
      Break;
  end;
  ConfFile.Free;
  if i > 0 then
    FKeyListCount := i;
end;

function TKeyTable.GetCmdId(Alt, Shift, Ctrl: Boolean; VKey: Integer): Integer;
var
  i: Integer;
begin
  for i := 0 to FKeyListCount - 1 do
  begin
    if (Alt = FKeyList[i].Alt) and (Shift = FKeyList[i].Shift)
      and (Ctrl = FKeyList[i].Ctrl) and (VKey = FKeyList[i].VKey) then
    begin
      Result := FKeyList[i].CmdId;
      Exit;
    end;
  end;
  Result := -1;
end;

function TKeyTable.StrToKey(const KeyName: string; const ActionName: string): TKey;
var
  i: Integer;
  key, s: string;
  p: PChar;
begin
  with Result do
  begin
    Alt := False;
    Shift := False;
    Ctrl := False;
    if Length(KeyName) = 1 then
      key := KeyName
    else begin
      s := KeyName;
      i := Pos('-', s);
      if i = 0 then
        key := KeyName
      else begin
        s[i] := #0;
        key := PChar(@s[i+1]);
        p := PChar(s);
        Alt := (Pos('A', p) > 0);
        Shift := (Pos('S', p) > 0);
        Ctrl := (Pos('C', p) > 0);
      end;
    end;

    VKey := -1;
    for i := 0 to High(SpKeys) do
    begin
      if key = SpKeys[i].Name then
      begin
        VKey := SpKeys[i].Id;
        Break;
      end;
    end;
    if VKey = -1 then
      VKey := Ord(key[1]); //VK_A;
    i := 0;
    while CmdTable[i].Name <> '' do
    begin
      if lstrcmpi(PChar(ActionName), PChar(CmdTable[i].Name)) = 0 then
      begin
        CmdId := CmdTable[i].Id;
        Exit;
      end;
      Inc(i);
    end;
    CmdId := -1;
  end;
end;

function TKeyTable.KeyToStr(const Key: TKey): string;
var
  i: Integer;
begin
  Result := '';
  if Key.Alt then Result := 'Alt+';
  if Key.Shift then Result := Result + 'Shift+';
  if Key.Ctrl then Result := Result + 'Ctrl+';
  for i := 0 to High(SpKeys) do
  begin
    if Key.Vkey = SpKeys[i].Id then
    begin
      Result := Result + SpKeys[i].Name;
      Exit;
    end;
  end;
  Result := Result + Char(Key.VKey);
end;

procedure TKeyTable.SetDefaultKeys(const DefaultKeys: array of string);
var
  i: Integer;
begin
  i := 0;
  FKeyListCount := 0;
  while DefaultKeys[i] <> '' do
  begin
    FKeyList[FKeyListCount] := StrToKey(DefaultKeys[i], DefaultKeys[i+1]);
    Inc(i, 2);
    Inc(FKeyListCount);
  end;
end;

end.
