unit SimpleList;

interface

type
  PPointerList = ^TPointerList;
  TPointerList = array[0..MaxInt div SizeOf(Pointer)-1] of Pointer;
  TSimpleListSortCompare = function (Item1, Item2: Pointer): Integer;
  TSimpleList = class(TObject)
  private
    FList: PPointerList;
    FCount: Integer;
    FCapacity: Integer;
  protected
    function Get(Index: Integer): Pointer;
    procedure Grow; virtual;
    procedure Put(Index: Integer; Item: Pointer);
    procedure SetCapacity(NewCapacity: Integer);
  public
    destructor Destroy; override;
    function Add(Item: Pointer): Integer;
    procedure Clear;
    procedure Delete(Index: Integer);
    property Count: Integer read FCount;
    property Items[Index: Integer]: Pointer read Get write Put; default;
    procedure Sort(Compare: TSimpleListSortCompare);
  end;

implementation

destructor TSimpleList.Destroy;
begin
  Clear;
end;

function TSimpleList.Add(Item: Pointer): Integer;
begin
  Result := FCount;
  if Result = FCapacity then
    Grow;
  FList^[Result] := Item;
  Inc(FCount);
end;

procedure TSimpleList.Clear;
begin
  FCount := 0; 
  SetCapacity(0);
end;

procedure TSimpleList.Delete(Index: Integer);
begin
  if (Index < 0) or (Index >= FCount) then
    Exit;
  Dec(FCount);
  if Index < FCount then
    Move(FList^[Index + 1], FList^[Index], (FCount - Index) * SizeOf(Pointer));
end;

function TSimpleList.Get(Index: Integer): Pointer;
begin
  if (Index < 0) or (Index >= FCount) then
    Result := nil
  else
    Result := FList^[Index];
end;

procedure TSimpleList.Grow;
var
  d: Integer;
begin
  if FCapacity > 64 then
    d := FCapacity div 4
  else if FCapacity > 8 then
    d := 16
  else
    d := 4;
  SetCapacity(FCapacity + d);
end;

procedure TSimpleList.Put(Index: Integer; Item: Pointer);
begin
  if (Index < 0) or (Index >= FCount) then
    Exit;
  FList^[Index] := Item;
end;

procedure TSimpleList.SetCapacity(NewCapacity: Integer);
begin
  if (NewCapacity < FCount) or (NewCapacity > (High(TPointerList)+1)) then
    Exit;
  if NewCapacity <> FCapacity then
  begin
    ReallocMem(FList, NewCapacity * SizeOf(Pointer));
    FCapacity := NewCapacity;
  end;
end;

procedure QuickSort(SortList: PPointerList; L, R: Integer;
  SCompare: TSimpleListSortCompare);
var
  I, J: Integer;
  P, T: Pointer;
begin
  repeat
    I := L;
    J := R;
    P := SortList^[(L + R) shr 1];
    repeat
      while SCompare(SortList^[I], P) < 0 do Inc(I);
      while SCompare(SortList^[J], P) > 0 do Dec(J);
      if I <= J then
      begin
        T := SortList^[I];
        SortList^[I] := SortList^[J];
        SortList^[J] := T;
        Inc(I);
        Dec(J);
      end;
    until I > J;
    if L < J then QuickSort(SortList, L, J, SCompare);
    L := I;
  until I >= R;
end;

procedure TSimpleList.Sort(Compare: TSimpleListSortCompare);
begin
  if (FList <> nil) and (FCount > 0) then
    QuickSort(FList, 0, FCount - 1, Compare);
end;

end.


